function entr = fuzzyen(ts, m, tau, r)
%FUZZYEN Fuzzy entropy
%       
%       Inputs:
%               ts     --- time-series
%               m      --- embedding dimension
%               tau    --- time delay
%               r      --- threshold value to determine similarity
% 
%       Ref.:
%       [1] Chen W, Zhuang J, Yu W, Wang Z. "Measuring complexity using
%           FuzzyEn, ApEn, and SampEn", Med Eng Phys 31: 61-68. 2009.
%       [2] Li P, Liu CY, Li K, Zheng D, Liu C, Hou Y. "Assessing the
%           complexity of short-term heartbeat interval series by
%           distribution entropy", Med Biol Eng Comput 53(1): 77-87. 2015.
%       [3] Shi B, Zhang Y, Yuan C, Wang S, Li P. "Entropy analysis of
%           short-term heartbeat interval time series during regular
%           walking", Entropy, 19(10): 568. 2017.
% 
% +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%                      (C) Peng Li 2015 -
% If you use the code, please make sure that you cite reference [1]-[3]
% +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
%
% $Author:      Peng Li
%               School of Control Science and Engineering,
%                   Shandong University, PR China
%                   Email: pli@sdu.edu.cn
% $Date:        Mar 23, 2015
% $Modif.:      
% 

% parse inputs
narginchk(4, 4);

% normalization
ts    = zscore(ts(:));

% reconstruction
N    = length(ts);
indm = hankel(1:N-m*tau, N-m*tau:N-tau);    % indexing elements for dim-m
inda = hankel(1:N-m*tau, N-m*tau:N);        % for dim-m+1
indm = indm(:, 1:tau:end);
inda = inda(:, 1:tau:end);
ym   = ts(indm);
ya   = ts(inda);

% using pdist for saving time, but a large RAM is required
cheb = pdist(ym, 'chebychev'); % inf-norm
cm   = sum(exp(-log(2).*(cheb./r).^2))*2 / (size(ym, 1)*(size(ym, 1)-1));

cheb = pdist(ya, 'chebychev');
ca   = sum(exp(-log(2).*(cheb./r).^2))*2 / (size(ya, 1)*(size(ya, 1)-1));

entr = -log(sum(ca) / sum(cm));